package com.nokia.task;
import com.jcraft.jsch.JSchException;
import com.nokia.common.gpload.GploadUtil;
import com.nokia.common.gpload.entity.GploadResult;
import com.nokia.common.ssh.SSHUtil;
import com.nokia.common.ssh.exception.SSHUtilException;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
@Slf4j
/*@Task*/
@Component
public class LtePmTask {

    @Value("${lte.pm.download.host:10.17.180.55}")
    private String host;
    @Value("${lte.pm.download.port:22}")
    private Integer port;
    @Value("${lte.pm.download.username:esbftp}")
    private String username;
    @Value("${lte.pm.download.password:Esb2019ftp!}")
    private String password;
    @Value("${lte.pm.download.sourceDir:/data/out2/pm_4g_hour}")
    private String sourceDir;
    @Value("${lte.pm.download.targetDir:download/}")
    private String downloadTargetDir;
    @Value("${lte.pm.filePrefix:pm_4g_hour_}")
    private String filePrefix;
    @Value("${lte.pm.distinct.targetDir:distinct/}")
    private String distinctTargetDir;


    @XxlJob("execHandler")
    public void xxlJobCall(){
        System.out.println("java-----------");
    }

    /**
     * 扫描文件
     */
    @XxlJob("")
    public void scan() throws JSchException, SSHUtilException, IOException {
        SSHUtil sshUtil = new SSHUtil(host, port, username, password);
        String exec = sshUtil.exec(sourceDir);
        File targetFile = new File(exec);
        String[] list = targetFile.list();
        for (String s : list) {
            String[] split = s.split("\\.");
            if (split[1].equals("csv")){
                singleTask(split[0]);
            }
        }
    }

/*    @AllowedTaskType({TaskType.CRON})
    public void cronTask(String hourDelay) throws JSchException, SSHUtilException, IOException {
        long hours = Long.parseLong(hourDelay);
        String hourString = dateFormat.format(new Date(System.currentTimeMillis() - 3600L * 1000L * hours));
        singleTask(hourString);
    }*/

    /*@AllowedTaskType({TaskType.TIMING, TaskType.IMMEDIATE})*/
    public void singleTask(String hourString) throws JSchException, SSHUtilException, IOException {
        download();
        distinct();
        gpload(hourString);
    }

    public void download() throws JSchException, SSHUtilException, IOException {
        SSHUtil sshUtil = new SSHUtil(host, port, username, password);
        String sourceFilePath = sourceDir;
        File targetFile = new File(downloadTargetDir);
        if (!targetFile.exists()) {
            targetFile.getParentFile().mkdirs();
        }
        String targetPath = targetFile.getAbsolutePath();
        boolean b = sshUtil.scpFrom(sourceFilePath, targetPath);
        if (b) {
            log.debug("文件 {} 下载成功...", targetPath);
            String exec = sshUtil.exec(sourceDir);
            File file = new File(exec);
            file.delete();
        }
    }

    public void distinct() throws IOException {
        String inputFilePath = downloadTargetDir ;
        String outputFilePath = distinctTargetDir;
        CSVFormat format = CSVFormat.DEFAULT.builder().build();
        CSVParser records = format.parse(new InputStreamReader(new FileInputStream(inputFilePath), StandardCharsets.UTF_8));
        Map<String, CSVRecord> recordMap = new HashMap<>();
        int count = 0;
        for (CSVRecord record : records) {
            recordMap.put(record.get(1), record);
            count++;
        }
        CSVPrinter printer = format.print(new File(outputFilePath), StandardCharsets.UTF_8);
        for (CSVRecord record : recordMap.values()) {
            printer.printRecord(record);
        }
        printer.flush();
        printer.close();
        log.debug("去重完成,原文件{}条数据,去重后{}条数据", count, recordMap.size());
    }

    public void gpload(String hourString) {
        String gploadCommand = "sh /data1/pm/gpload/pm_lte_gpload.sh " + hourString;
        GploadResult gpload = GploadUtil.gpload(gploadCommand);
        if (gpload.getTaskStatus()) {
            log.debug("gpload完成: {}", gpload);
        } else {
            log.error("gpload 失败: {}", gpload.getMessage());
        }
    }
}
